@@ -1,5 +1,5 @@
 #
-# Copyright 2013, Martin Owens <doctormo@gmail.com>
+# Copyright 2014, Martin Owens <doctormo@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 # REQUEST: Please do NOT simply copy and paste this code into your own
 #          projects. Please package this module for your distribution and
 #          use as a direct dependancy.
-# 
+#
 """
 from crontab import CronTab
 import sys
@@ -41,8 +41,8 @@ job.setall(1, 12, None, None, None)
 job2 = cron.new(command='/foo/bar',comment='SomeID')
 job2.every_reboot()
 
-list = cron.find_command('bar')
-job3 = list[0]
+jobs = list(cron.find_command('bar'))
+job3 = jobs[0]
 job3.clear()
 job3.minute.every(1)
 
@@ -66,9 +66,19 @@ for job7 in cron:
 
 cron.remove_all(command='/foo/bar')
 cron.remove_all(comment='This command')
+cron.remove_all(time='* * * * *')
 cron.remove_all()
+
+output = cron.render()
+
 cron.write()
 
+cron.write(filename='/tmp/output.txt')
+
+#cron.write_to_user(user=True)
+
+#cron.write_to_user(user='root')
+
 # Croniter Extentions allow you to ask for the scheduled job times, make
 # sure you have croniter installed, it's not a hard dependancy.
 
@@ -84,11 +94,11 @@ import subprocess as sp
 from datetime import date, datetime
 
 __pkgname__ = 'python-crontab'
-__version__ = '1.6.0'
+__version__ = '1.7.0'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)' +
     r'\s+([^@#\s]+)\s+([^@#\s]+)\s+([^#\n]*)(\s+#\s*([^\n]*)|$)')
-SPECREX = re.compile(r'@(\w+)\s([^#\n]*)(\s+#\s*([^\n]*)|$)')
+SPECREX = re.compile(r'^\s*@(\w+)\s([^#\n]*)(\s+#\s*([^\n]*)|$)')
 DEVNULL = ">/dev/null 2>&1"
 
 MONTH_ENUM = [ None,
@@ -162,15 +172,13 @@ class CronTab(object):
 
     """
     def __init__(self, user=None, tab=None, tabfile=None, log=None):
-        if user == True and not WINOS:
-            user = pwd.getpwuid( os.getuid() )[ 0 ]
         self.lines = None
         self.crons = None
         self.filen = None
         # Protect windows users
         self.root  = not WINOS and os.getuid() == 0
-        self.user  = user
-        # Detect older unixes and help them out.
+        self._user = user
+        # Load string or filename as inital crontab
         self.intab = tab
         self.read(tabfile)
         self._log = log
@@ -183,6 +191,12 @@ class CronTab(object):
             self._log = CronLog(self._log, user=self.user or 'root')
         return self._log
 
+    @property
+    def user(self):
+        if self._user == True and not WINOS:
+            return pwd.getpwuid( os.getuid() )[ 0 ]
+        return self._user
+
     def read(self, filename=None):
         """
         Read in the crontab from the system into the object, called
@@ -200,7 +214,9 @@ class CronTab(object):
             proc = sp.Popen(self._read_execute(),
                             stdout=sp.PIPE, stderr=sp.PIPE)
             (out, err) = proc.communicate()
-            if err:
+            if err and 'no crontab for' in err:
+                pass
+            elif err:
                 raise IOError("Read crontab %s: %s" % (self.user, err))
             lines = out.decode('utf-8').split("\n")
         for line in lines:
@@ -212,7 +228,7 @@ class CronTab(object):
                 self.lines.append(line.replace('\n',''))
 
     def write(self, filename=None):
-        """Write the crontab to the system. Saves all information."""
+        """Write the crontab to it's source or a given filename."""
         if filename:
             self.filen = filename
 
@@ -237,6 +253,14 @@ class CronTab(object):
             sp.Popen(self._write_execute(path)).wait()
             os.unlink(path)
 
+    def write_to_user(self, user=None):
+        """Write the crontab to a user (or root) instead of a file."""
+        self.filen = None
+        self.intab = None
+        if user != None:
+            self.user = user
+        self.write()
+
     def render(self):
         """Render this crontab as it would be in the crontab."""
         crons = []
@@ -259,20 +283,22 @@ class CronTab(object):
         return item
 
     def find_command(self, command):
-        """Return a list of crons using a command."""
-        result = []
-        for cron in self.crons:
-            if command in cron.command:
-                result.append(cron)
-        return result
+        """Return an iter of jobs matching any part of the command."""
+        for job in self.crons:
+            if command in job.command:
+                yield job
 
     def find_comment(self, comment):
-        """Return a list of crons using the comment field."""
-        result = []
-        for cron in self.crons:
-            if cron.comment == comment:
-                result.append(cron)
-        return result
+        """Return an iter of jobs that match the comment field exactly."""
+        for job in self.crons:
+            if job.comment == comment:
+                yield job
+
+    def find_time(self, *args):
+        """Return an iter of jobs that match this time pattern"""
+        for job in self.crons:
+            if job.slices == CronSlices(*args):
+                yield job
 
     @property
     def commands(self):
@@ -292,13 +318,15 @@ class CronTab(object):
                 yield cron.comment
                 returned.append(cron.comment)
 
-    def remove_all(self, command=None, comment=None):
+    def remove_all(self, command=None, comment=None, time=None):
         """Removes all crons using the stated command OR that have the
         stated comment OR removes everything if no arguments specified."""
         if command:
             return self.remove(*self.find_command(command))
         elif comment:
             return self.remove(*self.find_comment(comment))
+        elif time:
+            return self.remove(*self.find_time(time))
         return self.remove(*self.crons[:])
 
     def remove(self, *items):
@@ -360,8 +388,7 @@ class CronItem(object):
         self._log    = None
 
         # Initalise five cron slices using static info.
-        for i_value in range(0, 5):
-            self.slices.append(CronSlice(self, S_INFO[i_value]))
+        self.slices = CronSlices()
 
         if line and line.strip():
             self.parse(line.strip())
@@ -411,24 +438,9 @@ class CronItem(object):
         """Return true if this job is valid"""
         return self.valid
 
-    def render_time(self):
-        """Return just numbered parts of this crontab"""
-        return ' '.join([ unicode(self.slices[i]) for i in range(0, 5) ])
-
-    def render_schedule(self):
-        """Return just the first part of a cron job (the numbers or specials)"""
-        time = self.render_time()
-        if self.special:
-            return self.special
-        elif not SYSTEMV:
-            for (name, value) in SPECIALS.items():
-                if value == time and name not in SPECIAL_IGNORE:
-                    return "@%s" % name
-        return time
-
     def render(self):
         """Render this set cron-job to a string"""
-        result = "%s %s" % (self.render_schedule(), self.command)
+        result = "%s %s" % (str(self.slices), self.command)
         if self.comment:
             result += " # " + self.comment
         if not self.enabled:
@@ -438,7 +450,7 @@ class CronItem(object):
     def every_reboot(self):
         """Set to every reboot instead of a time pattern: @reboot"""
         self.clear()
-        self.special = '@reboot'
+        return self.slices.setall('@reboot')
 
     def every(self, unit=1):
         """
@@ -451,7 +463,7 @@ class CronItem(object):
         Many of these patterns exist as special tokens on Linux, such as
         `@midnight` and `@hourly`
         """
-        return SimpleItemInterface(self, unit)
+        return ItemEveryInterface(self.slices, unit)
 
     def setall(self, *args):
         """Replace existing time pattern with these five values given as args:
@@ -460,66 +472,34 @@ class CronItem(object):
            job.setall(1, 2) == '1 2 * * *'
            job.setall(0, 0, None, '>', 'SUN') == '0 0 * 12 SUN'
         """
-        if len(args) == 1 and isinstance(args[0], basestring):
-            if args[0].count(' ') == 4:
-                args = args[0].strip().split(' ')
-            elif args[0] == 'reboot':
-                self.special = '@'+args[0]
-                return True
-            elif args[0].strip()[0] == '@':
-                self.setall(SPECIALS.get(args[0][1:], args[0][1:]))
-
-        for x, s in enumerate(self.slices):
-            try:
-                s.parse(args[x])
-            except IndexError:
-                s.clear()
-            except KeyError:
-                return False
-        return True
+        return self.slices.setall(*args)
 
     def clear(self):
         """Clear the special and set values"""
-        self.special = None
-        for slice_v in self.slices:
-            slice_v.clear()
+        return self.slices.clear()
 
     def frequency(self, year=None):
         """Returns the number of times this item will execute in a given year
            (defaults to this year)
         """
-        return self.frequency_per_year() * self.frequency_per_day()
+        return self.slices.frequency(year=year)
 
     def frequency_per_year(self, year=None):
         """Returns the number of /days/ this item will execute on in a year
            (defaults to this year)
         """
-        result = 0
-        if not year:
-            year = date.today().year
-
-        weekdays = list(self[4])
-        days = (date(year+1,1,1) - date(year,1,1)).days
-
-        for month in self[3]:
-            for day in self[2]:
-                try:
-                    if date(year,month,day).weekday() in weekdays:
-                        result += 1
-                except ValueError:
-                    continue
-        return result
+        return self.slices.frequency_per_year(year=year)
 
     def frequency_per_day(self):
         """Returns the number of time this item will execute in any day"""
-        return len(self[0]) * len(self[1])
+        return self.slices.frequency_per_day()
 
     def schedule(self, date_from=None):
         """Return a croniter schedule is available."""
         if not date_from:
             date_from = datetime.now()
         if Croniter:
-            return Croniter(self.render_time(), date_from)
+            return Croniter(self.slices.clean_render(), date_from)
         raise ImportError("Croniter is not available. Please install croniter"+\
                          " python module via pip or your package manager")
 
@@ -583,8 +563,11 @@ class CronItem(object):
     def __getitem__(self, x):
         return self.slices[x]
 
-    def __eq__(self, value):
-        return str(self) == str(value)
+    def __lt__(self, value):
+        return self.frequency() < CronSlices(value).frequency()
+
+    def __gt__(self, value):
+        return self.frequency() > CronSlices(value).frequency()
 
     def __str__(self):
         return self.__unicode__()
@@ -596,7 +579,7 @@ class CronItem(object):
         return self.render()
 
 
-class SimpleItemInterface(object):
+class ItemEveryInterface(object):
     """Provide an interface to the job.every() method:
         Available Calls:
           minute, minutes, hour, hours, dom, doms, month, months, dow, dows
@@ -605,7 +588,7 @@ class SimpleItemInterface(object):
        will be set to '0' and the target unit will be set as every x units.
     """
     def __init__(self, item, units):
-        self.job = item
+        self.slices = item
         self.unit = units
         for (x, i) in enumerate(['minute', 'hour', 'dom', 'month', 'dow',
                                  'min', 'hour', 'day', 'moon', 'weekday']):
@@ -615,11 +598,11 @@ class SimpleItemInterface(object):
     def _set(self, target):
         def innercall():
             """Returned inner call for setting slice targets"""
-            self.job.clear()
+            self.slices.clear()
             # Day-of-week is actually a level 2 set, not level 4.
             for p in range(target == 4 and 2 or target):
-                self.job.slices[p].on('<')
-            self.job.slices[target].every(self.unit)
+                self.slices[p].on('<')
+            self.slices[target].every(self.unit)
         return innercall
 
     def year(self):
@@ -627,14 +610,111 @@ class SimpleItemInterface(object):
         if self.unit > 1:
             raise ValueError("Invalid value '%s', " % self.unit + \
                              "job may only be in '1' year.")
-        self.job.clear()
-        self.job.special = '@yearly'
+        self.slices.setall('@yearly')
+
+
+class CronSlices(list):
+    """Controls a list of five time 'slices' which reprisent:
+        minute frequency, hour frequency, day of month frequency,
+        month requency and finally day of the week frequency.
+     """
+    def __init__(self, *args):
+        for info in S_INFO:
+            self.append(CronSlice(info))
+        self.special = None
+        if args and not self.setall(*args):
+            raise ValueError("Can't set cron value to: %s" % str(args))
+
+    def setall(self, to, *slices):
+        """Parses the various ways date/time frequency can be specified"""
+        self.clear()
+        if isinstance(to, CronItem):
+            slices = to.slices
+        elif isinstance(to, list):
+            slices = to
+        elif slices:
+            slices = (to,) + slices
+        elif isinstance(to, basestring) and to:
+            if to.count(' ') == 4:
+                slices = to.strip().split(' ')
+            elif to.strip()[0] == '@':
+                to = to[1:]
+            else:
+                slices = [to]
+
+            if to == 'reboot':
+                self.special = '@reboot'
+                return True
+            elif to in SPECIALS.keys():
+                return self.setall(SPECIALS[to])
+
+        if id(slices) == id(self):
+            raise ValueError("Can not set cron to itself!")
+
+        for a, b in zip(self, slices):
+            try:
+                a.parse(b)
+            except Exception:
+                return False
+        return True
+
+    def clean_render(self):
+        """Return just numbered parts of this crontab"""
+        return ' '.join([ unicode(s) for s in self ])
+
+    def render(self):
+        """Return just the first part of a cron job (the numbers or specials)"""
+        time = self.clean_render()
+        if self.special:
+            return self.special
+        elif not SYSTEMV:
+            for (name, value) in SPECIALS.items():
+                if value == time and name not in SPECIAL_IGNORE:
+                    return "@%s" % name
+        return time
+
+    def clear(self):
+        """Clear the special and set values"""
+        self.special = None
+        for s in self:
+            s.clear()
+
+    def frequency(self, year=None):
+        return self.frequency_per_year(year=year) * self.frequency_per_day()
+
+    def frequency_per_year(self, year=None):
+        result = 0
+        if not year:
+            year = date.today().year
+
+        weekdays = list(self[4])
+        days = (date(year+1,1,1) - date(year,1,1)).days
+
+        for month in self[3]:
+            for day in self[2]:
+                try:
+                    if date(year,month,day).weekday() in weekdays:
+                        result += 1
+                except ValueError:
+                    continue
+        return result
+
+    def frequency_per_day(self):
+        """Returns the number of time this item will execute in any day"""
+        return len(self[0]) * len(self[1])
+
+    def __str__(self):
+        return self.render()
+
+    def __eq__(self, arg):
+        return self.render() == CronSlices(arg).render()
+
+
 
 
 class CronSlice(object):
     """Cron slice object which shows a time pattern"""
-    def __init__(self, job, info, value=None):
-        self.job   = job
+    def __init__(self, info, value=None):
         self.name  = info.get('name', None)
         self.min   = info.get('min', None)
         self.max   = info.get('max', None)
@@ -646,6 +726,8 @@ class CronSlice(object):
     def parse(self, value):
         """Set values into the slice."""
         self.parts = []
+        if value == None:
+            return self.clear()
         for part in str(value).split(','):
             if part.find("/") > 0 or part.find("-") > 0 or part == '*':
                 self.parts.append( self.get_range( part ) )
@@ -728,6 +810,9 @@ class CronSlice(object):
     def __iter__(self):
         """Return the entire element as an iterable"""
         r = {}
+        # An empty part means '*' which is every(1)
+        if not self.parts:
+            self.every(1)
         for part in self.parts:
             if isinstance(part, CronRange):
                 for bit in part.range():
@@ -753,7 +838,7 @@ class CronSlice(object):
         except KeyError:
             raise KeyError("No enumeration '%s' got '%s'" % (self.name, v))
 
-        # If it's a sunday, make it a sunday
+        # If it's a sunday, make it a sunday (force 7 to 0)
         if self.max == 6 and v == 7:
             out = 0
 
@@ -858,7 +943,7 @@ class CronRange(object):
 
     def range(self):
         """Returns the range of this cron slice as a iterable list"""
-        return range(self.vfrom, self.vto+1, self.seq)
+        return range(int(self.vfrom), int(self.vto)+1, self.seq)
 
     def every(self, value):
         """Set the sequence value for this range."""
